@@ -532,12 +532,8 @@ close_current (GtkMenuItem * mi, gpointer user_data)
 	            False,
 	            SubstructureRedirectMask | SubstructureNotifyMask,
 	            &xev);
-  gdk_flush ();
-#if GTK_CHECK_VERSION(3, 0, 0)
+	gdk_flush ();
 	gdk_error_trap_pop_ignored ();
-#else
-	gdk_error_trap_pop ();
-#endif
 
 	return;
 }
